from PodSix.Resource import *
from PodSix.GUI.Widget import Widget

class ToolTip(Widget):
	"""
	Singleton for tooltip widget help.
	"""
	def __init__(self, font="default", color=[0, 0, 0]):
		Widget.__init__(self)
		self.font = font
		self.color = color
		self.bg = (255, 255, 255)
		self.widget = None
		# should be always on top of everything
		self.priority = 100
		self.pos = None
		self.oldpos = None
		self.rect = None
		self.delay_time = 0
		self.moved = None
	
	def __call__(self, widget):
		"""
		set new widget
		"""
		self.widget = hasattr(widget, "help_text") and widget or None
		self.delay_time = 1
	
	def Draw(self):
		if not self.widget:
			return
		pos = gfx.GetMouse()
		if not self.widget.InRect(pos):
			self.widget = None
			self.rect = None
			return
		
		self.delay_time -= self.Elapsed()
		if self.delay_time <= 0:
			if not pos == self.oldpos:
				b = 10.0 / gfx.width
				centerx, centery = gfx.ClampTextToScreen((float(pos[0]) / gfx.width, 
									  float(pos[1] - 8) / gfx.width),
									 self.widget.help_text,
									 self.font, border=(b,b))
				self.pos = {"centerx": centerx, "centery": centery}
				self.rect = gfx.DrawText(self.widget.help_text, self.pos, self.color, self.font)
				self.oldpos = pos
		
			gfx.DrawRect([self.rect.left - 3, self.rect.top - 3, self.rect.width + 7, self.rect.height + 7], self.bg, 0)
			gfx.DrawLines([(self.rect.left - 3, self.rect.top - 4), (self.rect.right + 4, self.rect.top - 4)], self.color, False)
			gfx.DrawLines([(self.rect.left - 3, self.rect.bottom + 4), (self.rect.right + 4, self.rect.bottom + 4)], self.color, False)
			gfx.DrawLines([(self.rect.left - 4, self.rect.top - 3), (self.rect.left - 4, self.rect.bottom + 4)], self.color, False)
			gfx.DrawLines([(self.rect.right + 4, self.rect.top - 3), (self.rect.right + 4, self.rect.bottom + 4)], self.color, False)
			self.rect = gfx.DrawText(self.widget.help_text, self.pos, self.color, self.font)
		elif not pos == self.moved:
			self.delay_time = 1
		self.moved = pos

tooltip = ToolTip()
