from PodSix.Concurrent import Concurrent
from PodSix.Rectangle import Rectangle
from PodSix.RDC import RDCEntity

from Collider import Collider
from Platform import Platform
from Ladder import Ladder


class Character(Concurrent, Collider):
	def __init__(self, rectangle=None):
		Concurrent.__init__(self)
		self.rectangle = Rectangle(rectangle)
		self.hspeed = 0.5
		self.jump = 1.2
		self.level = None
		self.animation = "stand.right"
		self.velocity = [0, 0]
		self.platform = None
		self.ladder = None
		# last platfomr or ladder
		self.lastplatform = None
		self.grabbing = False
		self.lastmove = "StopRight"
	
	def Collide(self, who):
		if isinstance(who, Platform):
			myr = self.rectangle
			thr = who.rectangle
			hitplatform = False
			# figure out which side has penetrated the least
			sides = [
					[thr.Top() - myr.Bottom(), 1, 1],
					[thr.Left() - myr.Right(), 0, 1],
					[myr.Left() - thr.Right(), 0, -1],
					[myr.Top() - thr.Bottom(), 1, -1]
			]
			sides.sort(lambda a, b: cmp(b[0], a[0]))
			d = sides[0]
			
			# if we've hit a platform underneath us and don't want to climb on a ladder
			if (d[1] == 1 and int(d[2]) == 1 and self.platform != who) and not self.ladder:
				self.platform = who
				hitplatform = True
			
			# move to the outside of the platform on the least penetrated side
			self.rectangle[d[1]] += d[2] * d[0]
			# stop moving if moving toward the platform in the vertical
			if d[1] and d[2] == cmp(self.velocity[d[1]], 0):
				self.velocity[d[1]] = 0
			
			if hitplatform:
				self.HitPlatform(who)
		
		elif isinstance(who, Ladder) and self.ladder is None and self.grabbing:
			if self.platform:
				self.LeavePlatform()
			self.ladder = who
			# what to do with velocity?
			# set vertical speed for moving up, as the player pressed
			# the up-key (but before grabbing the ladder so it had
			# no effect). Otherwise you have to press the up key 
			# again. This is an annoing delay, especially if you are
			# are standing on a platform and want to climp up.
			#self.velocity = [0, -self.hspeed]
			self.velocity[1] = -self.hspeed
	
	def Update(self):
		self.rectangle.CenterY(self.rectangle.CenterY() + self.velocity[1] * self.Elapsed(0.1))
		self.rectangle.CenterX(self.rectangle.CenterX() + self.velocity[0] * self.Elapsed(0.1))
		
		Concurrent.Update(self)
		
		# if you fall off a platform, run leaveplatform
		if self.platform and (self.velocity[1] > 0.05 or self.velocity[1] < 0):
			self.LeavePlatform()
		
		# update the animations for freefall
		if self.platform:
			if self.velocity[0]:
				self.animation = "walk." + self.animation.split(".")[1]
			else:
				self.animation = "stand." + self.animation.split(".")[1]
		
		if self.ladder:
			if not self.ladder.Touches(self.rectangle):
				if self.grabbing:
					# hack
					self.ladder = None
					self.platform = True
					self.Jump()
				else:
					self.LeavePlatform()
	
	def HitPlatform(self, platform):
		""" Override me to do something when the user lands on a platform. """
		pass
	
	def SetLevel(self, level):
		self.level = level
		self.platform = None
		self.lastplatform = None
	
	def StopRight(self, force=False):
		self.lastmove = "StopRight"
		if self.animation == "walk.right" or force:
			self.animation = "stand.right"
		if self.animation.split(".")[1] == "right":
			self.velocity[0] = 0
	
	def StopLeft(self, force=False):
		self.lastmove = "StopLeft"
		if self.animation == "walk.left" or force:
			self.animation = "stand.left"
		if self.animation.split(".")[1] == "left":
			self.velocity[0] = 0
	
	def WalkRight(self, force=False):
		self.lastmove = "WalkRight"
		self.animation = self.animation.split(".")[0] + ".right"
		self.velocity[0] = self.hspeed
	
	def WalkLeft(self, force=False):
		self.lastmove = "WalkLeft"
		self.animation = self.animation.split(".")[0] + ".left"
		self.velocity[0] = -self.hspeed
	
	def Jump(self, force=False):
		self.lastmove = "Jump"
		if self.platform or self.ladder:
			self.velocity[1] = -self.jump
			self.Update()
			self.LeavePlatform()
		self.animation = "jump." + self.animation.split(".")[1]
	
	def LeavePlatform(self):
		if self.platform or self.ladder:
			self.lastplatform = self.platform or self.ladder
			self.platform = None
			self.ladder = None
		#self.animation = "jump." + self.animation.split(".")[1]
	
	def ClimbUp(self, force=False):
		#self.animation = 
		self.lastmove = "ClimbUp"
		self.grabbing = True
		if self.ladder:
			self.velocity[1] = -self.hspeed
			return True
		else:
			return False
	
	def ClimbDown(self, force=False):
		self.lastmove = "ClimbDown"
		#self.animation = 
		if self.ladder:
		    self.velocity[1] = self.hspeed
	
	def StopUp(self, force=False):
		self.lastmove = "StopUp"
		self.grabbing = False
#		if self.animation == "???" or force:
#			self.animation = "???"
#		if self.animation.split(".")[1] == "???":
		if self.ladder:
		    self.velocity[1] = 0
		
	def StopDown(self, force=False):
		self.lastmove = "StopDown"
#		if self.animation == "???" or force:
#			self.animation = "???"
#		if self.animation.split(".")[1] == "???":
		if self.ladder:
		    self.velocity[1] = 0
	
	def Say(self, text):
		pass

