class RectangleException(Exception):
	pass

class Rectangle(list):
	"""
		This is basically a copy of pygame's "rect" class, but slower and in pure Python.
	"""
	def __init__(self, vals=[0, 0, 0, 0]):
		if len(vals) != 4:
			raise RectangleException("Rectangle must have four values")
		else:
			list.__init__(self, vals)
	
	def Set(self, *args):
		self.__init__(*args)
	
	def PointInRect(self, pos):
		return pos[0] < self.Right() and pos[0] > self.Left() and pos[1] < self.Bottom() and pos[1] > self.Top()
	
	def Width(self, val=None):
		if not val is None:
			self[2] = val
		return self[2]
	
	def Height(self, val=None):
		if not val is None:
			self[3] = val
		return self[3]
	
	def Right(self, val=None):
		if not val is None:
			self[0] = val - self[2]
		return self[0] + self[2]
	
	def Left(self, val=None):
		if not val is None:
			self[0] = val
		return self[0]
	
	def Top(self, val=None):
		if not val is None:
			self[1] = val
		return self[1]
	
	def Bottom(self, val=None):
		if not val is None:
			self[1] = val - self[3]
		return self[1] + self[3]
	
	def Position(self, val=None):
		if not val is None:
			self[:2] = val
		return self[:2]
	
	def CenterX(self, val=None):
		if not val is None:
			self[0] = val - self[2] / 2
		return self[0] + self[2] / 2
	
	def CenterY(self, val=None):
		if not val is None:
			self[1] = val - self[3] / 2
		return self[1] + self[3] / 2
	
	def Center(self, val=None):
		if not val is None:
			if len(val) == 2:
				self[0] = val[0] - self[2] / 2
				self[1] = val[1] - self[3] / 2
			else:
				raise RectangleException("Can only set Center to a two value list")
		return [self[0] + self[2] / 2, self[1] + self[3] / 2]
	
	def Grow(self, *amount):
		self[2] += amount[0]
		self[3] += amount[1]
		return self
	
	def Absolute(self):
		""" Rectify negative width/height """
		for x in (2, 3):
			if self[x] < 0:
				self[x - 2] += self[x]
				self[x] = -self[x]
		return self
	
	def SetBounds(self, x1, y1, x2, y2):
		self.Left(x1)
		self.Width(x2 - x1)
		self.Top(y1)
		self.Height(y2 - y1)
		return self
	
	def Clip(self, rectangle):
		"""
		Returns a version of the current rectangle clipped by the second rectangle.
		Retains negative width and/or height in the current rectangle if the original contained negative width and/or height.
		
		>>> Rectangle([10, 10, 5, 5]).Clip([8, 8, 4, 4])
		[10, 10, 2, 2]
		>>> Rectangle([8, 8, 4, 4]).Clip([10, 10, 5, 5])
		[10, 10, 2, 2]
		>>> Rectangle([5, 5, 10, 10]).Clip([6, 6, 4, 4])
		[6, 6, 4, 4]
		>>> Rectangle([6, 6, 4, 4]).Clip([5, 5, 10, 10])
		[6, 6, 4, 4]
		"""
		new = [max(rectangle[0], self[0]), max(rectangle[1], self[1])]
		new2 = [min(rectangle[0] + rectangle[2], self[0] + self[2]) - new[0], min(rectangle[1] + rectangle[3], self[1] + self[3]) - new[1]]
		return Rectangle(new + new2)

def _test():
	import doctest
	doctest.testmod()

if __name__ == "__main__":
	_test()

